/*
 *  sysdeps.h - System dependent definitions for PlayStation Portable
 *
 *  Basilisk II (C) 1997-2001 Christian Bauer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef SYSDEPS_H
#define SYSDEPS_H

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspdisplay.h>

// Setup endianess and addressing.
#define WORDS_BIGENDIAN 0
#define REAL_ADDRESSING 0
#define DIRECT_ADDRESSING 1

// Using 68k emulator
#define EMULATED_68K 1

// Mac ROM is write protected
#define ROM_IS_WRITE_PROTECTED 1

// ExtFS is not supported (yet)
#define SUPPORTS_EXTFS 0

// mon is not supported
#undef ENABLE_MON

// Time data type for Time Manager emulation
struct tm_time_t { int tv_sec, tv_usec; };

// Offset Mac->PSP time in seconds (TODO: Fixme)
#define TIME_OFFSET 0x7c25b080

// RAM Amount [16MB] (TODO: Move to a prefs file)
#define RAM_AMOUNT (16 * (1024 * 1024))

// Paths (TODO: Move to a prefs file)
#define ROM_PATH "ms0:/PSP/GAME/BasiliskIIPSP/mac.rom"
#define XPRAM_FILE_NAME "ms0:/PSP/GAME/BasiliskIIPSP/BasiliskII.xpram"
#define PREFS_FILE_NAME "ms0:/PSP/GAME/BasiliskIIPSP/BasiliskII_Prefs"
#define BOOT_DISK_FILE_NAME "ms0:/PSP/GAME/BasiliskIIPSP/system.dsk"

// Use scratch mem
#define USE_SCRATCHMEM_SUBTERFUGE 1

// Data types
typedef unsigned char uint8;
typedef signed char int8;
typedef unsigned short uint16;
typedef signed short int16;
typedef unsigned long uint32;
//typedef signed long int32;
typedef unsigned long long uint64;
//typedef signed long long int64;
typedef uint32 uintptr;
typedef unsigned long long loff_t;

// UAE CPU data types
#define uae_s8 int8
#define uae_u8 uint8
#define uae_s16 int16
#define uae_u16 uint16
#define uae_s32 int32
#define uae_u32 uint32
#define uae_s64 int64
#define uae_u64 uint64
typedef uae_u32 uaecptr;
#define VAL64(a) (a ## LL)
#define UVAL64(a) (a ## uLL)

#define printf pspDebugScreenPrintf
#define ENUMDECL typedef enum
#define ENUMNAME(name) name
#define REGPARAM2
#define REGPARAM 
#define ASM_SYM_FOR_FUNC(a) __asm__(a)
#define write_log pspDebugScreenPrintf
#define CPU_EMU_SIZE 0

#define htons(A)	((((uint16)(A) & 0xff00) >> 8) | \
			 (((uint16)(A) & 0x00ff) << 8))
#define htonl(A)	((((uint32)(A) & 0xff000000) >> 24) | \
			 (((uint32)(A) & 0x00ff0000) >> 8)  | \
			 (((uint32)(A) & 0x0000ff00) << 8)  | \
			 (((uint32)(A) & 0x000000ff) << 24))
#define ntohs htons
#define ntohl htonl

#define do_get_mem_byte(a) ((uae_u32)*((uae_u8 *)(a)))
#define do_put_mem_byte(a, v) (*(uae_u8 *)(a) = (v))

#define ftruncate
#define call_mem_get_func(func, addr) ((*func)(addr))
#define call_mem_put_func(func, addr, v) ((*func)(addr, v))

// Other little-endian CPUs which can not do unaligned accesses (this needs optimization)
static inline uae_u32 do_get_mem_long(uae_u32 *a) {uint8 *b = (uint8 *)a; return (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];}
static inline uae_u32 do_get_mem_word(uae_u16 *a) {uint8 *b = (uint8 *)a; return (b[0] << 8) | b[1];}
static inline void do_put_mem_long(uae_u32 *a, uae_u32 v) {uint8 *b = (uint8 *)a; b[0] = v >> 24; b[1] = v >> 16; b[2] = v >> 8; b[3] = v;}
static inline void do_put_mem_word(uae_u16 *a, uae_u32 v) {uint8 *b = (uint8 *)a; b[0] = v >> 8; b[1] = v;}
#endif
