/*
 *  video_psp.cpp - Video/graphics emulation, PlayStation Portable specific stuff
 *
 *  Basilisk II (C) 1997-2001 Christian Bauer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sysdeps.h"
#include "main.h"
#include "adb.h"
#include "macos_util.h"
#include "prefs.h"
#include "user_strings.h"
#include "video.h"
#include "../uae_cpu/memory.h"
#include "../uae_cpu/cpu_emulation.h"

#include <pspgu.h>

#define DEBUG 0
#include "debug.h"

// Video setup
#define FB_TYPE				u16
#define PSP_MODE 			GU_PSM_5650
#define MAC_MODE			FLAYOUT_HOST_565
							//FLAYOUT_DIRECT
#define PIXEL_SIZE			2
#define SCREEN_WIDTH		480
#define SCREEN_HEIGHT		272
#define PSP_LINE_SIZE		512
#define MAC_COLOR_MODE		VMODE_16BIT
#define PSP_COLOR_MODE		PSP_DISPLAY_PIXEL_FORMAT_565
#define FRAMEBUFFER_SIZE	(PSP_LINE_SIZE * SCREEN_HEIGHT * PIXEL_SIZE)
FB_TYPE *g_vram_base = (FB_TYPE *)(0x40000000 | 0x04000000);
static unsigned int __attribute__((aligned(16))) list[256];
static int dispBufferNumber;
inline FB_TYPE *getVramDrawBuffer()
{
	return ((!dispBufferNumber) ? g_vram_base + (FRAMEBUFFER_SIZE >> 1) : g_vram_base);
}

// Basilisk display configuration
static FB_TYPE __attribute__((aligned(16))) PspFrameBuffer[SCREEN_WIDTH * SCREEN_HEIGHT];

/*
 *  Initialization
 */

bool VideoInit(bool classic)
{
#if !DEBUG
	// guInit - Taken from initGraphics() in graphics.c from LUAPlayer
        sceDisplaySetMode(0, SCREEN_WIDTH, SCREEN_HEIGHT);

        dispBufferNumber = 0;
        sceDisplayWaitVblankStart();
		sceDisplaySetFrameBuf((void*) g_vram_base, PSP_LINE_SIZE, PSP_COLOR_MODE, 1);

        sceGuInit();

        sceGuStart(GU_DIRECT, list);
        sceGuDrawBuffer(PSP_MODE, (void*)FRAMEBUFFER_SIZE, PSP_LINE_SIZE);
        sceGuDispBuffer(SCREEN_WIDTH, SCREEN_HEIGHT, (void*)0, PSP_LINE_SIZE);
        sceGuClear(GU_COLOR_BUFFER_BIT | GU_DEPTH_BUFFER_BIT);
        sceGuDepthBuffer((void*) 0x110000, PSP_LINE_SIZE);
        sceGuOffset(2048 - (SCREEN_WIDTH / 2), 2048 - (SCREEN_HEIGHT / 2));
        sceGuViewport(2048, 2048, SCREEN_WIDTH, SCREEN_HEIGHT);
        sceGuDepthRange(0xc350, 0x2710);
        sceGuScissor(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
        sceGuEnable(GU_SCISSOR_TEST);
        sceGuAlphaFunc(GU_GREATER, 0, 0xff);
        sceGuEnable(GU_ALPHA_TEST);
        sceGuDepthFunc(GU_GEQUAL);
        sceGuEnable(GU_DEPTH_TEST);
        sceGuFrontFace(GU_CW);
        sceGuShadeModel(GU_SMOOTH);
        sceGuEnable(GU_CULL_FACE);
        sceGuEnable(GU_TEXTURE_2D);
        sceGuTexMode(PSP_MODE, 0, 0, 0);
        sceGuTexFunc(GU_TFX_REPLACE, GU_TCC_RGBA);
        sceGuTexFilter(GU_NEAREST, GU_NEAREST);
        sceGuAmbientColor(0xffffffff);
        sceGuFinish();
        sceGuSync(0, 0);

        sceDisplayWaitVblankStart();
        sceGuDisplay(1);
#endif
	// Configure the Basilisk II framebuffer.
	VideoMonitor.x = SCREEN_WIDTH;
	VideoMonitor.y = SCREEN_HEIGHT;
	VideoMonitor.mode = MAC_COLOR_MODE;
	VideoMonitor.bytes_per_row = SCREEN_WIDTH * PIXEL_SIZE;
	VideoMonitor.mac_frame_base = MacFrameBaseMac;

	MacFrameBaseHost = (uint8 *)PspFrameBuffer;
	MacFrameSize = VideoMonitor.bytes_per_row * VideoMonitor.y;
	MacFrameLayout = MAC_MODE;

	// Done.
	D(bug("video "));
	return true;
}

void video_set_palette(unsigned char *p)
{
	D(bug("video_set_palette called\n"));
}

/*
 *  Deinitialization
 */

void VideoExit(void)
{
#if !DEBUG
	sceGuTerm();
#endif
}


/*
 *  Video message handling
 */

void VideoInterrupt(void)
{
#if !DEBUG
/*
	// Color correction
	FB_TYPE *FBE = PspFrameBuffer + (SCREEN_WIDTH * SCREEN_HEIGHT * PIXEL_SIZE);
	for (FB_TYPE *FBB = PspFrameBuffer; FBB < FBE; FBB++)
		*FBB = (((*FBB & 0x00FF) << 8) | ((*FBB & 0xFF00) >> 8));
*/
	// blitImageToScreen(...) with some modifications.
	FB_TYPE* vram = getVramDrawBuffer();
	sceKernelDcacheWritebackInvalidateAll();
	sceGuStart(GU_DIRECT,list);
	sceGuCopyImage(PSP_MODE, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT,
		SCREEN_WIDTH, PspFrameBuffer, 0, 0, PSP_LINE_SIZE, vram);
	sceGuFinish();
	sceGuSync(0,0);
	
	// flipScreen()
	sceGuSwapBuffers();
	sceDisplaySetFrameBuf(vram, PSP_LINE_SIZE, PSP_COLOR_MODE, 1);
	dispBufferNumber ^= 1;
#endif
}
