/*
 *  xpram_psp.cpp - XPRAM handling, PlayStation Portable implementation
 *
 *  Basilisk II (C) 1997-2001 Christian Bauer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sysdeps.h"

#include <stdio.h>
#include <stdlib.h>

#include "xpram.h"
#include "debug.h"

/*
 *  Load XPRAM from settings file
 */

void LoadXPRAM(void)
{
	D(bug("Reading XPRAM... "));
	int fd = sceIoOpen(XPRAM_FILE_NAME, PSP_O_RDONLY, 0777);
	if (fd < 0) { D(bug("failed.\n")); return; }
	sceIoRead(fd, XPRAM, 256);
	sceIoClose(fd);
	D(bug("success.\n"));
}


/*
 *  Save XPRAM to settings file
 */

void SaveXPRAM(void)
{
	int fd = sceIoOpen(XPRAM_FILE_NAME, PSP_O_CREAT | PSP_O_TRUNC | PSP_O_WRONLY, 0777);
	if (fd < 0) return;
	
	bool ok = true;
	if (sceIoWrite(fd, XPRAM, 256) != 256) ok = false;
	if (sceIoClose(fd) != 0) ok = false;
	if (!ok) D(bug("Error writing XPRAM. Memory stick may be full."));
}


/*
 *  Delete PRAM file
 */

void ZapPRAM(void)
{
	sceIoRemove(XPRAM_FILE_NAME);
}
